@@ -1,4 +1,4 @@
-/* $Id: gdict-app.c,v 1.124 2005/03/03 17:08:30 vnoel Exp $ */
+/* $Id: gdict-app.c,v 1.130 2005/05/25 15:01:40 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -147,7 +147,8 @@ gdict_lookup_definition (GDictWindow *gdict, gchar *text)
   gint retval;
   gchar *word_entry_text;
 
-  g_return_if_fail (text != NULL);
+	if (text == NULL)
+		return;
 
   /* If needed update the the word_entry */
   word_entry_text = gtk_editable_get_chars (GTK_EDITABLE (gdict->word_entry), 0, -1);
@@ -160,9 +161,8 @@ gdict_lookup_definition (GDictWindow *gdict, gchar *text)
 
   g_free (word_entry_text);
 
-  if (gdict->pref->smart && is_pattern (text)) {
+  if (is_pattern (text))
     retval = gdict_lookup_spelling (gdict, text, TRUE);
-  }
   else {
     retval = gdict_defbox_lookup (gdict->defbox, text);
     if (!retval)
@@ -512,18 +512,30 @@ gdict_web_search_cb (GtkAction *action, GtkWidget *widget, gpointer data)
   if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menu_item))) {
     gdict->search_type = SEARCH_TYPE_REGULAR;
     gtk_widget_show (gdict->paned);
-    gtk_widget_show (gdict->statusbar);
     gtk_window_set_resizable (GTK_WINDOW (gdict), TRUE);
     gtk_window_resize (GTK_WINDOW (gdict), width, height);
   } else {
     gdict->search_type = SEARCH_TYPE_WEB;
     gtk_widget_hide (gdict->paned);
-    gtk_widget_hide (gdict->statusbar);
     gtk_window_get_size (GTK_WINDOW (gdict), &width, &height);
     gtk_window_set_resizable (GTK_WINDOW (gdict), FALSE);
   }
 }
 
+static void
+gdict_zoom_in (GtkAction *action, GtkWidget *widget)
+{
+	GDictWindow *gdict = GDICT_WINDOW (widget);
+	gdict_defbox_zoom_in (gdict->defbox);
+}
+
+static void
+gdict_zoom_out (GtkAction *action, GtkWidget *widget)
+{
+	GDictWindow *gdict = GDICT_WINDOW (widget);
+	gdict_defbox_zoom_out (gdict->defbox);
+}
+
 static void
 gdict_about_cb (GtkAction *action, GtkWidget *widget)
 {
@@ -574,12 +586,17 @@ gdict_window_new (void)
   gdict->search_type = SEARCH_TYPE_REGULAR;
   gdict->web_search_sites = gdict_web_search_default_sites ();
   gdict_windows = g_slist_prepend (gdict_windows, window);
+	if (gdict->pref->fontname) {
+		gdict_defbox_set_font (gdict->defbox, gdict->pref->fontname);
+		gdict_defbox_set_fontsize (gdict->defbox, gdict->pref->fontsize);
+	} else
+		gdict_window_store_font (gdict);
   return window;
 }
 
 /* Menu bar */
 
-static const gchar *ui_info = 
+static const gchar ui_info[] = 
 "<ui>"
 "  <menubar name='MenuBar'>"
 "    <menu action='DictionaryMenu'>"
@@ -602,6 +619,9 @@ static const gchar *ui_info =
 "    </menu>"
 "    <menu action='ViewMenu'>"
 "      <menuitem action='ViewSpellings'/>"
+"      <separator/>"
+"      <menuitem action='ViewZoomIn'/>"
+"      <menuitem action='ViewZoomOut'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='HelpContents'/>"
@@ -610,7 +630,7 @@ static const gchar *ui_info =
 "  </menubar>"
 "</ui>";
 
-static const gchar *search_type_ui = 
+static const gchar search_type_ui[] = 
 "<ui>"
 "<menubar name='SearchTypeMenuBar'>"
 " <menu action='SearchTypeMenu'>"
@@ -620,7 +640,7 @@ static const gchar *search_type_ui =
 "</menubar>"
 "</ui>";
 
-static GtkActionEntry menu_entries[] = {
+static const GtkActionEntry menu_entries[] = {
 
   /* Top */
   { "DictionaryMenu", NULL, N_("_Dictionary") },
@@ -644,6 +664,12 @@ static GtkActionEntry menu_entries[] = {
   { "Preferences", GTK_STOCK_PREFERENCES, N_("P_references"), NULL,
     N_("Configure the application"), G_CALLBACK (gdict_preferences_cb) },
 
+	/* View menu */
+	{"ViewZoomIn", GTK_STOCK_ZOOM_IN, NULL, "<control>plus", N_("Bigger text size"),
+	 G_CALLBACK (gdict_zoom_in)},
+	{"ViewZoomOut", GTK_STOCK_ZOOM_OUT, NULL, "<control>minus", N_("Smaller text size"),
+	 G_CALLBACK (gdict_zoom_out)},
+
   /* Help menu */
   { "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1",
      N_("View help for this application"), G_CALLBACK (gdict_help_contents_cb) },
@@ -651,8 +677,7 @@ static GtkActionEntry menu_entries[] = {
     N_("About this application"), G_CALLBACK (gdict_about_cb) }
 };
 
-
-static GtkRadioActionEntry search_type_entries[] = {
+static const GtkRadioActionEntry search_type_entries[] = {
   { "SearchTypeMenu", NULL, "Search Type"},
   { "RegularSearch", NULL, N_("On a Dictionary Server"), NULL,
     NULL, 1},
@@ -661,7 +686,7 @@ static GtkRadioActionEntry search_type_entries[] = {
 };
 
 /* a separate group for items that change sensitivity */
-static GtkActionEntry edit_entries[] = {
+static const GtkActionEntry edit_entries[] = {
   { "Cut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X",
     N_("Cut the selection"), G_CALLBACK (cut_cb) },
   { "Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
@@ -673,11 +698,69 @@ static GtkActionEntry edit_entries[] = {
 };
 
 /* a seperate group for toggle entries */
-static GtkToggleActionEntry toggle_entries[] = {
+static const GtkToggleActionEntry toggle_entries[] = {
   { "ViewSpellings", GTK_STOCK_SPELL_CHECK, N_("_Spellings"), "<control>S",
     N_("View alternate spellings"), G_CALLBACK (gdict_view_spellings_cb) }
 };
 
+static void
+menu_item_select_cb (GtkMenuItem *proxy, gpointer data)
+{
+  GDictWindow *gdict = data;
+  GtkAction *action;
+  char *message;
+
+  action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+  g_return_if_fail (action != NULL);
+
+  g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+  if (message)
+    {
+      gtk_statusbar_push (GTK_STATUSBAR (gdict->statusbar),
+                          gdict->tip_message_cid, message);
+      g_free (message);
+    }
+}
+
+static void
+menu_item_deselect_cb (GtkMenuItem *proxy, gpointer data)
+{
+  GDictWindow *gdict = data;
+
+  gtk_statusbar_pop (GTK_STATUSBAR (gdict->statusbar),
+                     gdict->tip_message_cid);
+}
+
+static void
+connect_proxy_cb (GtkUIManager *manager,
+                  GtkAction *action,
+                  GtkWidget *proxy,
+                  gpointer data)
+{
+  if (GTK_IS_MENU_ITEM (proxy))
+    {
+      g_signal_connect (proxy, "select",
+                  G_CALLBACK (menu_item_select_cb), data);
+      g_signal_connect (proxy, "deselect",
+                  G_CALLBACK (menu_item_deselect_cb), data);
+    }
+}
+
+static void
+disconnect_proxy_cb (GtkUIManager *manager,
+                     GtkAction *action,
+                     GtkWidget *proxy,
+                     gpointer data)
+{
+  if (GTK_IS_MENU_ITEM (proxy))
+    {
+      g_signal_handlers_disconnect_by_func (proxy,
+                  G_CALLBACK (menu_item_select_cb), data);
+      g_signal_handlers_disconnect_by_func (proxy,
+                  G_CALLBACK (menu_item_deselect_cb), data);
+    }
+}
+
 GtkWidget *
 gdict_window_init (GDictWindow *gdict)
 {
@@ -733,7 +816,7 @@ gdict_window_init (GDictWindow *gdict)
   gtk_ui_manager_insert_action_group (ui, gdict->edit_actions, 0);
   gtk_ui_manager_insert_action_group (ui, gdict->search_type_actions, 0);
   gtk_window_add_accel_group (GTK_WINDOW (window), 
-            gtk_ui_manager_get_accel_group (ui));
+  gtk_ui_manager_get_accel_group (ui));
 
   if (!gtk_ui_manager_add_ui_from_string (ui, ui_info, -1, &error)) {
     g_message ("building menus failed: %s", error->message);
@@ -742,6 +825,12 @@ gdict_window_init (GDictWindow *gdict)
   }
   gtk_ui_manager_add_ui_from_string (ui, search_type_ui, -1, &error);
 
+  /* show tooltips in the statusbar */
+  g_signal_connect (ui, "connect_proxy",
+              G_CALLBACK (connect_proxy_cb), gdict);
+  g_signal_connect (ui, "disconnect_proxy",
+              G_CALLBACK (disconnect_proxy_cb), gdict);
+
   container = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), container);
 
@@ -833,6 +922,7 @@ gdict_window_init (GDictWindow *gdict)
 
   gtk_widget_show_all (container);
   gtk_widget_hide (GTK_WIDGET(speller));
+	gtk_widget_hide (gdict->statusbar);
   return GTK_WIDGET(gdict);
 }
 
