@@ -1,4 +1,4 @@
-/* $Id: gdict-speller.c,v 1.31 2004/11/24 22:15:27 vnoel Exp $ */
+/* $Id: gdict-speller.c,v 1.43 2005/05/19 22:33:54 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -16,14 +16,8 @@
 #  include <config.h>
 #endif
 
-#include <string.h>
-
-#include <gnome.h>
-#ifdef HAVE_GNOME_PRINT
-#  include <libgnomeprint/gnome-print.h>
-#  include <libgnomeprint/gnome-printer-dialog.h>
-#  include <math.h>
-#endif
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include "gdict-speller.h"
 #include "gdict-pref.h"
@@ -42,7 +36,7 @@ enum {
 
 static gint signals[LAST_SIGNAL] = { 0, 0, 0 };
 
-static GtkDialogClass *parent_class;
+static GtkVBoxClass *parent_class;
 
 static void gdict_speller_init (GDictSpeller *speller);
 static void gdict_speller_class_init (GDictSpellerClass *class);
@@ -63,53 +57,15 @@ gdict_speller_get_type (void)
 			(GInstanceInitFunc) gdict_speller_init
 		};
 
-		type = g_type_register_static (GTK_TYPE_DIALOG,
+		type = g_type_register_static (GTK_TYPE_VBOX,
 					       "GDictSpeller", &info, 0);
 	}
 
 	return type;
 }
 
-/* speller_lookup_cb
- *
- * Looks up the currently selected word
- */
 static void
-speller_lookup (GDictSpeller *speller)
-{
-	if (!speller->current_word)
-		return;
-
-	gtk_entry_set_text (GTK_ENTRY (word_entry), speller->current_word);
-	gtk_editable_select_region (GTK_EDITABLE (word_entry), 0,
-				    strlen (speller->current_word));
-	gdict_app_do_lookup (speller->current_word);
-}
-
-static void
-spell_button_pressed (GtkButton *button, GDictSpeller *speller)
-{
-	gchar *text;
-
-	g_return_if_fail (GDICT_IS_SPELLER (speller));
-
-	text = gtk_editable_get_chars (GTK_EDITABLE (speller->word_entry), 0, -1);
-	gtk_editable_select_region (GTK_EDITABLE (speller->word_entry), 0,
-				    strlen (text));
-	gdict_speller_lookup (speller, text);
-	g_free (text);
-}
-
-static void
-spell_entry_cb (GtkEntry *entry, GDictSpeller *speller)
-{
-	g_return_if_fail (GDICT_IS_SPELLER (speller));
-
-	spell_button_pressed (NULL, speller);
-}
-
-static void
-row_selected_cb (GtkTreeSelection *selection, GDictSpeller *speller)
+gdict_speller_row_selected_cb (GtkTreeSelection *selection, GDictSpeller *speller)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -123,42 +79,25 @@ row_selected_cb (GtkTreeSelection *selection, GDictSpeller *speller)
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (speller->word_list));
 	gtk_tree_model_get (model, &iter, 0, &word, -1);
 	speller->current_word = g_strdup (word);
+
+	gdict_lookup_definition (speller->gdict, word);
 }
 
 static void
-row_activated_cb (GtkTreeView *tree,
-		  GtkTreePath *path,
-		  GtkTreeViewColumn *column,
-		  GDictSpeller *speller)
+gdict_speller_hide (GtkButton *button, gpointer data)
 {
-	g_return_if_fail (GDICT_IS_SPELLER (speller));
-
-	speller_lookup (speller);
-}
-
-static void
-spell_response_cb (GtkDialog *dialog, gint id, GDictSpeller *speller)
-{
-	g_return_if_fail (GDICT_IS_SPELLER (speller));
-
-	switch (id) {
-	case GTK_RESPONSE_DELETE_EVENT:
-	case GTK_RESPONSE_CLOSE:
-		gtk_widget_hide (GTK_WIDGET (dialog));
-		break;
-	case GDICT_SPELL_RESPONSE:
-		speller_lookup (speller);
-		break;
-	default:
-		break;
-	}
+	GDictSpeller *speller = data;
+	GDictWindow *gdict = speller->gdict;
+	GtkWidget *check;
+	check = gtk_ui_manager_get_widget (gdict->ui, "/MenuBar/ViewMenu/ViewSpellings");
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (check), FALSE);
 }
 
 static void 
 gdict_speller_init (GDictSpeller *speller)
 {
 	GtkWidget *label, *button, *scrolled_win;
-	GtkWidget *hbox, *expander;
+	GtkWidget *hbox;
 	GtkWidget *vbox;
 	GtkListStore *model;
 	GtkTreeViewColumn *column;
@@ -166,76 +105,38 @@ gdict_speller_init (GDictSpeller *speller)
 	GtkCellRenderer *cell;
 
 	speller->context = NULL;
-	speller->get_strat_cmd = NULL;
 	speller->spell_cmd = NULL;
-	speller->strat = gdict_pref.dfl_strat;
 
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-
-	hbox = gtk_hbox_new (FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-    
-	label = gtk_label_new_with_mnemonic (_("_Word:"));
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-    
-	speller->word_entry = GTK_ENTRY (gtk_entry_new ());
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label),
-				       GTK_WIDGET (speller->word_entry));
-
-	gtk_box_pack_start (GTK_BOX (hbox),
-			    GTK_WIDGET (speller->word_entry), TRUE, TRUE, 0);
-	g_signal_connect (G_OBJECT (speller->word_entry), "activate",
-			  G_CALLBACK (spell_entry_cb), speller);
-
-	button = gdict_button_new_with_stock_image (_("Check _Spelling"),
-						    GTK_STOCK_SPELL_CHECK);
+	vbox = GTK_WIDGET(&(speller->vbox));
+	gtk_box_set_spacing (GTK_BOX (vbox), 3);
+	
+	hbox = gtk_hbox_new (FALSE, 0);
+	label = gtk_label_new (_("Spellings"));
+	gtk_misc_set_alignment (GTK_MISC(label), 0., 0.5);
+	gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);
+	button = gdict_button_new_with_stock_image (NULL, GTK_STOCK_CLOSE);
+	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (button), "clicked",
-			  G_CALLBACK (spell_button_pressed), speller);
+			  G_CALLBACK (gdict_speller_hide), speller);
 
-	if (gail_loaded) {
-		gdict_add_atk_namedesc (label, _("Word"), _("Word"));
-		gdict_add_atk_namedesc (GTK_WIDGET(speller->word_entry),
-					_("Word Entry"), _("Enter a word to know the spelling"));
-	}
-
-	speller->hbox = gtk_hbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (speller->hbox), 6);
-	speller->strat_label = gtk_label_new_with_mnemonic (_("Search s_trategy:"));
-	gtk_box_pack_start (GTK_BOX (speller->hbox),
-			    speller->strat_label, FALSE, FALSE, 0);
-
-	expander = gtk_expander_new_with_mnemonic (_("Show more _options"));
-	gtk_container_add (GTK_CONTAINER (expander), speller->hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), expander, FALSE, FALSE, 0);
-
-	if (gail_loaded)
-        	gdict_add_atk_namedesc (GTK_WIDGET (button),
-					NULL, _("Click to do the spell check"));
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
 	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (scrolled_win));
-
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
 					GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_win),
 					     GTK_SHADOW_ETCHED_IN);
 
-	gtk_widget_set_size_request (scrolled_win, 400, 250);
-
 	model = gtk_list_store_new (1, G_TYPE_STRING);
 
 	speller->word_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (speller->word_list), FALSE);
 	g_object_unref (G_OBJECT (model));
     
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label),
-				       GTK_WIDGET (speller->word_list));
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (speller->word_list));
 	g_signal_connect (G_OBJECT (selection), "changed",
-			  G_CALLBACK (row_selected_cb), speller);
+			  G_CALLBACK (gdict_speller_row_selected_cb), speller);
 
 	cell = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes ("words", cell, "text", 0, NULL);
@@ -246,27 +147,7 @@ gdict_speller_init (GDictSpeller *speller)
 	gtk_container_add (GTK_CONTAINER (scrolled_win), speller->word_list);
 	gtk_box_pack_start (GTK_BOX (vbox), scrolled_win, TRUE, TRUE, 0);
 
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (speller)->vbox), vbox);
-
-	gtk_dialog_add_button (GTK_DIALOG (speller), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-	gdict_dialog_add_button (GTK_DIALOG (speller), _("_Look Up Word"),
-				 GTK_STOCK_FIND, GDICT_SPELL_RESPONSE);
-
-	gtk_dialog_set_default_response (GTK_DIALOG (speller), GDICT_SPELL_RESPONSE);
-	g_signal_connect (G_OBJECT (speller->word_list), "row_activated",
-			  G_CALLBACK (row_activated_cb), speller);
-	g_signal_connect (G_OBJECT (speller), "response", 
-			  G_CALLBACK (spell_response_cb), speller);
-
-	/* To prevent dialog destruction when pressing ESC */
-	g_signal_connect (G_OBJECT (speller), "delete_event", 
-			  G_CALLBACK (gtk_true), NULL);
-
-	gtk_window_set_title (GTK_WINDOW (speller), _("Check Spelling"));
-	gtk_dialog_set_has_separator (GTK_DIALOG (speller), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (speller), 0);
-
-	gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (speller)->vbox));
+	gtk_widget_show (GTK_WIDGET (speller));
 }
 
 static void
@@ -279,7 +160,6 @@ gdict_speller_finalize (GObject *object)
 	speller = GDICT_SPELLER (object);
 
 	g_free (speller->database);
-	dict_command_destroy (speller->get_strat_cmd);
 	dict_command_destroy (speller->spell_cmd);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -333,25 +213,72 @@ gdict_speller_class_init (GDictSpellerClass *class)
 	parent_class = g_type_class_peek_parent (class);
 	class->word_lookup_done = NULL;
 	class->word_not_found = NULL;
-
+	
 	object_class->finalize = gdict_speller_finalize;
 }
 
+static void
+gdict_speller_set_sensitive (GDictWindow *gdict, gboolean sensitive)
+{
+  GtkAction *action;
+  gtk_widget_set_sensitive (gdict->speller, sensitive);
+  action = gtk_ui_manager_get_action (gdict->ui, "/MenuBar/ViewMenu/ViewSpellings");
+  gtk_action_set_sensitive (action, sensitive);
+}
+
+static void
+gdict_speller_lookup_start_cb (GtkWidget *widget, gpointer data) 
+{
+  GDictWindow *gdict = data;
+	gtk_widget_show (gdict->statusbar);
+  gtk_statusbar_pop (GTK_STATUSBAR(gdict->statusbar), 0);
+  gtk_statusbar_push (GTK_STATUSBAR(gdict->statusbar), 0, _("Spell-checking..."));
+  gdict_speller_set_sensitive (gdict, FALSE);
+}
+
+static void
+gdict_speller_lookup_done_cb (GtkWidget *widget, gpointer data) 
+{
+  GDictWindow *gdict = data;
+	gtk_widget_hide (gdict->statusbar);
+  gdk_window_set_cursor (GTK_WIDGET(gdict)->window, NULL);
+  gdict_speller_set_sensitive (gdict, TRUE);
+}
+
+static void
+gdict_speller_not_found_cb (GtkWidget *widget, gpointer data) 
+{
+  GDictWindow *gdict = data;
+	gtk_widget_show (gdict->statusbar);
+  gtk_statusbar_pop (GTK_STATUSBAR (gdict->statusbar), 0);
+  gtk_statusbar_push (GTK_STATUSBAR (gdict->statusbar), 0, _("No matches found"));
+  gdk_window_set_cursor (GTK_WIDGET(gdict)->window, NULL);
+  gdict_speller_set_sensitive (gdict, TRUE);
+}
+
 GtkWidget *
-gdict_speller_new ()
+gdict_speller_new (GDictWindow *gdict)
 {
 	GtkWidget *widget;
+  GDictSpeller *speller;
 
 	widget = g_object_new (GDICT_TYPE_SPELLER, NULL);
-
+	speller = GDICT_SPELLER (widget);
+  speller->gdict = gdict;
+  g_signal_connect (speller, "word_lookup_start",
+        G_CALLBACK (gdict_speller_lookup_start_cb), gdict);
+  g_signal_connect (speller, "word_lookup_done",
+        G_CALLBACK (gdict_speller_lookup_done_cb), gdict);
+  g_signal_connect (speller, "word_not_found",
+        G_CALLBACK (gdict_speller_not_found_cb), gdict);
+  g_signal_connect (speller, "socket_error",
+        G_CALLBACK (gdict_socket_error_cb), gdict);
 	return widget;
 }
 
 static void
-spell_error_cb (dict_command_t *command,
-		DictStatusCode code,
-                gchar *message,
-		gpointer data)
+gdict_speller_error_cb (dict_command_t *command, DictStatusCode code,
+                gchar *message, gpointer data)
 {
 	GtkWindow *speller;
 
@@ -370,8 +297,7 @@ spell_error_cb (dict_command_t *command,
 		gtk_widget_destroy (dialog);
 
 		if (command->cmd == C_MATCH)
-			g_signal_emit (G_OBJECT (speller), 
-				       signals[WORD_LOOKUP_DONE_SIGNAL], 0);
+			g_signal_emit (G_OBJECT (speller), signals[WORD_LOOKUP_DONE_SIGNAL], 0);
 	}
 	else {
 		g_signal_emit (G_OBJECT (speller),
@@ -379,12 +305,42 @@ spell_error_cb (dict_command_t *command,
 	}
 }
 
-/* spell_word_status_cb
+void
+gdict_speller_change_db (GDictSpeller *speller, gchar *new_db)
+{
+	GDictWindow *gdict = speller->gdict;
+	gchar *text;
+
+	g_free (speller->database);
+	speller->database = g_strdup_printf ("%s", new_db);
+	text = gtk_editable_get_chars (GTK_EDITABLE (gdict->word_entry), 0, -1);
+	gtk_editable_select_region (GTK_EDITABLE (gdict->word_entry), 0,
+				    strlen (text));
+	gdict_speller_lookup (speller->gdict, speller, text);
+    	g_free (text);
+}
+
+void
+gdict_speller_change_strategy (GDictSpeller *speller, gchar *new_strategy)
+{
+	GDictWindow *gdict = speller->gdict;
+	gchar *text;
+
+	g_free (speller->strat);   
+	speller->strat = g_strdup_printf ("%s", new_strategy);
+	text = gtk_editable_get_chars (GTK_EDITABLE (gdict->word_entry), 0, -1);
+	gtk_editable_select_region (GTK_EDITABLE (gdict->word_entry), 0,
+				    strlen (text));
+	gdict_speller_lookup (speller->gdict, speller, text);
+    	g_free (text);
+}
+	
+/* gdict_speller_word_status_cb
  *
  * Callback used when a status code has arrived over the link
  */
 static void 
-spell_word_status_cb (dict_command_t *command,
+gdict_speller_word_status_cb (dict_command_t *command,
 		      DictStatusCode code, 
 		      int num_found,
 		      gpointer data)
@@ -411,20 +367,21 @@ spell_word_status_cb (dict_command_t *command,
  * Sets the current search strategy to the one indicated
  */
 static void
-speller_set_strat_cb (GtkWidget *widget, GDictSpeller *speller)
+gdict_speller_set_strat_cb (GtkWidget *widget, GDictSpeller *speller)
 {
 	gchar *text;
 	gchar *strat;
+	GDictWindow *gdict = speller->gdict;
 
 	g_return_if_fail (GDICT_IS_SPELLER (speller));
 
 	strat = g_object_get_data (G_OBJECT (widget), "strat_name");
 	speller->strat = strat;
 
-	text = gtk_editable_get_chars (GTK_EDITABLE (speller->word_entry), 0, -1);
-	gtk_editable_select_region (GTK_EDITABLE (speller->word_entry), 0,
+	text = gtk_editable_get_chars (GTK_EDITABLE (gdict->word_entry), 0, -1);
+	gtk_editable_select_region (GTK_EDITABLE (gdict->word_entry), 0,
 				    strlen (text));
-	gdict_speller_lookup (speller, text);
+	gdict_speller_lookup (speller->gdict, speller, text);
     	g_free (text);
 }
 
@@ -436,10 +393,11 @@ void
 gdict_speller_clear (GDictSpeller *speller)
 {
 	GtkTreeModel *model;
+	GDictWindow *gdict = speller->gdict;
 
 	g_return_if_fail (GDICT_IS_SPELLER (speller));
     
-	gtk_entry_set_text (speller->word_entry, "");
+	gtk_entry_set_text (GTK_ENTRY(gdict->word_entry), "");
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (speller->word_list));
 	gtk_list_store_clear (GTK_LIST_STORE (model));
 	speller->current_word = NULL;
@@ -451,7 +409,7 @@ gdict_speller_clear (GDictSpeller *speller)
 }
 
 static void 
-speller_add_word (GDictSpeller *speller, gchar *word)
+gdict_speller_add_word (GDictSpeller *speller, gchar *word)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -465,12 +423,12 @@ speller_add_word (GDictSpeller *speller, gchar *word)
 		speller->current_word = word;
 }
 
-/* spell_word_data_cb
+/* gdict_speller_word_data_cb
  *
  * Callback used when a new word has arrived over the link
  */
 static void 
-spell_word_data_cb (dict_command_t *command,
+gdict_speller_word_data_cb (dict_command_t *command,
 		    dict_res_t *res,
 		    gpointer data)
 {
@@ -483,7 +441,7 @@ spell_word_data_cb (dict_command_t *command,
 	if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (speller)))
 		return;
 
-	speller_add_word (speller, res->desc);
+	gdict_speller_add_word (speller, res->desc);
 }
 
 /* gdict_speller_lookup
@@ -495,12 +453,17 @@ spell_word_data_cb (dict_command_t *command,
  * Returns 0 on success and -1 on command invocation error
  */
 gint 
-gdict_speller_lookup (GDictSpeller *speller, gchar *text)
+gdict_speller_lookup (GDictWindow *gdict, GDictSpeller *speller, gchar *text)
 {
 	g_return_val_if_fail (speller != NULL, -1);
 	g_return_val_if_fail (GDICT_IS_SPELLER (speller), -1);
 	g_return_val_if_fail (text != NULL, -1);
 
+	if (speller->context == NULL) {		
+		gdict_init_context (gdict);
+		speller->context = gdict->context;
+	}
+
 	while (isspace (*text))
 		text++;
 
@@ -512,17 +475,17 @@ gdict_speller_lookup (GDictSpeller *speller, gchar *text)
 
 	gdict_speller_clear (speller);
     
-	gtk_entry_set_text (speller->word_entry, text);
-	gtk_editable_select_region (GTK_EDITABLE (speller->word_entry), 
+	gtk_entry_set_text (GTK_ENTRY(gdict->word_entry), text);
+	gtk_editable_select_region (GTK_EDITABLE (gdict->word_entry), 
 				    0, strlen (text));
 
 	g_free (speller->database);
-	speller->database = g_strdup (gdict_pref.database);
+	speller->database = g_strdup (gdict->pref->database);
     
 	speller->spell_cmd = dict_match_command_new (speller->database, speller->strat, text);
-	speller->spell_cmd->error_notify_cb = spell_error_cb;
-	speller->spell_cmd->status_notify_cb = spell_word_status_cb;
-	speller->spell_cmd->data_notify_cb = spell_word_data_cb;
+	speller->spell_cmd->error_notify_cb = gdict_speller_error_cb;
+	speller->spell_cmd->status_notify_cb = gdict_speller_word_status_cb;
+	speller->spell_cmd->data_notify_cb = gdict_speller_word_data_cb;
 	speller->spell_cmd->user_data = speller;
 
 	if (dict_command_invoke (speller->spell_cmd, speller->context) < 0)
@@ -539,160 +502,24 @@ void
 gdict_speller_reset (GDictSpeller *speller, dict_context_t *context)
 {
 	gchar *word;
+        GDictWindow *gdict = speller->gdict;
     
 	/* If we have a new context, we cannot trust that the search strategy
 	 * we were using will still be present */
-	if (context != speller->context) {
-		speller->context = context;
-		speller->strat = gdict_pref.dfl_strat;
-		gdict_speller_reset_strat (speller);
+	if (gdict->context != speller->context) {
+		speller->context = gdict->context;
+		speller->strat = gdict->pref->dfl_strat;
 	}
 
 	/* Re-invoke current query only if there is a new database specified */
 	if (context != speller->context ||
-	    strcmp (speller->database, gdict_pref.database)) {
+	    strcmp (speller->database, gdict->pref->database)) {
 		if (speller->spell_cmd) {
 			word = g_strdup (speller->spell_cmd->search_term);
 			dict_command_destroy (speller->spell_cmd);
 			speller->spell_cmd = NULL;
-			gdict_speller_lookup (speller, word);
+			gdict_speller_lookup (speller->gdict, speller, word);
 			g_free (word);
 		}
 	}
 }
-
-/* gdict_speller_get_word
- *
- * Returns the word defined in the speller, if any
- */
-gchar *
-gdict_speller_get_word (GDictSpeller *speller)
-{
-	g_return_val_if_fail (GDICT_IS_SPELLER (speller), NULL);
-
-	return speller->spell_cmd ? speller->spell_cmd->search_term : NULL;
-}
-
-static void
-speller_add_strat (GDictSpeller *speller, gchar *strat, gchar *desc)
-{
-	GtkWidget *menu_item;
-	GtkWidget *label;
-
-	label = gtk_label_new (desc);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-
-	menu_item = gtk_menu_item_new ();
-	g_signal_connect (menu_item, "activate", 
-			  G_CALLBACK (speller_set_strat_cb), speller);
-	g_object_set_data (G_OBJECT (menu_item), "strat_name", strat);
-	gtk_container_add (GTK_CONTAINER (menu_item), label);
-	gtk_widget_show_all (menu_item);
-	gtk_menu_append (speller->strat_list, menu_item);
-
-	if (!strcmp (speller->strat, strat))
-		gtk_menu_set_active (speller->strat_list, speller->strat_idx);
-
-	speller->strat_idx++;
-}
-
-static void
-spell_strat_data_cb (dict_command_t *command,
-		     dict_res_t *res,
-		     gpointer data)
-{
-	GDictSpeller *speller;
-
-	g_return_if_fail (GDICT_IS_SPELLER (data));
-
-	speller = GDICT_SPELLER (data);
-
-	if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (speller)))
-		return;
-
-	speller_add_strat (speller, res->name, res->desc);
-}
-
-/* spell_strat_status_cb
- *
- * Callback used when a status code has arrived over the link
- */
-static void 
-spell_strat_status_cb (dict_command_t *command,
-		       DictStatusCode code, 
-		       int num_found,
-		       gpointer data)
-{
-	GDictSpeller *speller;
-	GtkWidget *alignment;
-
-	g_return_if_fail (GDICT_IS_SPELLER (data));
-
-	speller = GDICT_SPELLER (data);
-
-	if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (speller)))
-		return;
-
-	if (code == DICT_STATUS_OK) {
-		speller->strat_sel = GTK_OPTION_MENU (gtk_option_menu_new ());
-		gtk_option_menu_set_menu (speller->strat_sel, 
-					  GTK_WIDGET (speller->strat_list));
-		alignment = gtk_alignment_new (0, 0.5, 0, 0);
-		gtk_container_add (GTK_CONTAINER (alignment), 
-				   GTK_WIDGET (speller->strat_sel));
-		gtk_box_pack_start (GTK_BOX (speller->hbox), alignment, TRUE, TRUE, 0);
-		gtk_widget_show_all (GTK_WIDGET (speller->hbox));
-
-		gtk_label_set_mnemonic_widget (GTK_LABEL (speller->strat_label),
-					       GTK_WIDGET (speller->strat_sel));        
-
-		if (gail_loaded) {
-			gdict_add_atk_namedesc (speller->strat_label,
-					        _("Search Strategy"), _("Search Strategy"));
-			gdict_add_atk_relation (GTK_WIDGET (speller->strat_sel),
-						speller->strat_label, ATK_RELATION_LABELLED_BY);
-		}
-	}
-}
-
-void
-gdict_speller_reset_strat (GDictSpeller *speller)
-{
-	GtkWidget *error_label, *alignment;
-
-	g_return_if_fail (GDICT_IS_SPELLER (speller));
-
-	if (speller->get_strat_cmd)
-		dict_command_destroy (speller->get_strat_cmd);
-
-	if (speller->strat_sel) {
-		gtk_option_menu_remove_menu (speller->strat_sel);
-		gtk_widget_destroy (GTK_WIDGET (speller->strat_sel));
-		gtk_widget_show_all (GTK_WIDGET (speller->hbox));
-		speller->strat_sel = NULL;
-	}
-    
-	speller->strat_list = GTK_MENU (gtk_menu_new ());
-    
-	speller->get_strat_cmd = dict_show_strat_command_new ();
-	speller->get_strat_cmd->error_notify_cb = spell_error_cb;
-	speller->get_strat_cmd->data_notify_cb = spell_strat_data_cb;
-	speller->get_strat_cmd->status_notify_cb = spell_strat_status_cb;
-	speller->get_strat_cmd->user_data = speller;
-	speller->strat_idx = 0;
-    
-	if (dict_command_invoke (speller->get_strat_cmd,
-				 speller->context) == -1) 
-	{
-		/* Could not look up search strategies, so just display a
-		 * label; FIXME: Memory leak */
-		error_label = gtk_label_new (_("Cannot connect to server"));
-		alignment = gtk_alignment_new (0, 0.5, 0, 0);
-		gtk_container_add (GTK_CONTAINER (alignment), 
-				   GTK_WIDGET (error_label));
-		gtk_box_pack_start (GTK_BOX (speller->hbox), alignment, TRUE, TRUE, 0);
-		gtk_widget_show_all (GTK_WIDGET (speller->hbox));
-	}
-}
-
