@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.46 2005/01/24 17:54:32 vnoel Exp $ */
+/* $Id: main.c,v 1.47 2005/04/02 07:20:01 dcransto Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -87,7 +87,7 @@ int main (int argc, char *argv[])
 				      GNOME_PARAM_APP_DATADIR,
 				      DATADIR, NULL);
 
-	g_set_application_name (_("Gnome Dictionary"));
+	g_set_application_name (_("Dictionary"));
 	gtk_window_set_default_icon_name ("gdict");
 
 	if ((client = gnome_master_client()) != NULL) {
