@@ -32,12 +32,13 @@
 #include <glib/gi18n.h>
 #include <regex.h>
 #include <gdk/gdkx.h>
-#include <bonobo-activation/bonobo-activation.h>
+#include <libart_lgpl/art_rgb.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-ops.h> 
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnome/gnome-desktop-item.h>
+#include <libgnomeui/gnome-thumbnail.h>
 
 #include <gnome.h>
 
@@ -130,7 +131,7 @@ gsearchtool_gconf_get_list (const gchar * key)
 	g_return_val_if_fail (key != NULL, FALSE);
 	
 	client = gsearchtool_gconf_client_get_global ();
-	g_return_val_if_fail (client != NULL, FALSE);
+	g_return_val_if_fail (client != NULL, NULL);
 	
 	result = gconf_client_get_list (client, key, GCONF_VALUE_STRING, &error);
 	
@@ -141,6 +142,27 @@ gsearchtool_gconf_get_list (const gchar * key)
 	return result;
 }
 
+gint 
+gsearchtool_gconf_get_int (const gchar * key)
+{
+	GConfClient * client;
+	GError * error = NULL;
+	gint result;
+
+	g_return_val_if_fail (key != NULL, FALSE);
+	
+	client = gsearchtool_gconf_client_get_global ();
+	g_return_val_if_fail (client != NULL, FALSE);
+	
+	result = gconf_client_get_int (client, key, &error);
+	
+	if (gsearchtool_gconf_handle_error (&error)) {
+		result = 0;
+	}
+	
+	return result;
+}
+
 gboolean 
 gsearchtool_gconf_get_boolean (const gchar * key)
 {
@@ -216,6 +238,7 @@ is_quick_search_excluded_path (const gchar * path)
 	GSList     * exclude_path_list;
 	GSList     * tmp_list;
 	gchar      * dir;
+	gboolean     results = FALSE;
 	
 	dir = g_strdup (path);
 	
@@ -239,22 +262,21 @@ is_quick_search_excluded_path (const gchar * path)
 	
 		gchar * dir;
 		
-		dir = g_strdup (tmp_list->data);
-		
 		/* Skip empty or null values. */
-		if ((dir == NULL) || (strlen (dir) == 0)) {
+		if ((tmp_list->data == NULL) || (strlen (tmp_list->data) == 0)) {
 			continue;
 		}
+		
+		dir = g_strdup (tmp_list->data);
 			
 		/* Wild-card comparisons. */		
 		if (g_strstr_len (dir, strlen (dir), "*") != NULL) { 
 		
 			if (g_pattern_match_simple (dir, path) == TRUE) {
 
-				g_slist_free (exclude_path_list);
+				results = TRUE;
 				g_free (dir);			
-
-				return TRUE;
+				break;
 			}
 		} 
 		/* Non-wild-card comparisons. */
@@ -271,17 +293,20 @@ is_quick_search_excluded_path (const gchar * path)
 			
 			if (strcmp (path, dir) == 0) {
 				
-				g_slist_free (exclude_path_list);
+				results = TRUE;
 				g_free (dir);
-				
-				return TRUE;
+				break;
 			}
 		}
 		g_free (dir);
 	}
+	
+	for (tmp_list = exclude_path_list; tmp_list; tmp_list = tmp_list->next) {
+		g_free (tmp_list->data);
+	}
 	g_slist_free (exclude_path_list);
 	
-	return FALSE;
+	return results;
 }
 
 gboolean  	
@@ -290,6 +315,7 @@ is_second_scan_excluded_path (const gchar * path)
 	GSList     * exclude_path_list;
 	GSList     * tmp_list;
 	gchar      * dir;
+	gboolean     results = FALSE;
 	
 	dir = g_strdup (path);
 	
@@ -313,22 +339,21 @@ is_second_scan_excluded_path (const gchar * path)
 	
 		gchar * dir;
 		
-		dir = g_strdup (tmp_list->data);
-		
 		/* Skip empty or null values. */
-		if ((dir == NULL) || (strlen (dir) == 0)) {
+		if ((tmp_list->data == NULL) || (strlen (tmp_list->data) == 0)) {
 			continue;
 		}
+
+		dir = g_strdup (tmp_list->data);
 			
 		/* Wild-card comparisons. */		
 		if (g_strstr_len (dir, strlen (dir), "*") != NULL) { 
 		
 			if (g_pattern_match_simple (dir, path) == TRUE) {
 
-				g_slist_free (exclude_path_list);
-				g_free (dir);			
-
-				return TRUE;
+				results = TRUE;
+				g_free (dir);
+				break;
 			}
 		} 
 		/* Non-wild-card comparisons. */
@@ -345,17 +370,20 @@ is_second_scan_excluded_path (const gchar * path)
 			
 			if (strcmp (path, dir) == 0) {
 				
-				g_slist_free (exclude_path_list);
+				results = TRUE;
 				g_free (dir);
-				
-				return TRUE;
+				break;
 			}
 		}
 		g_free (dir);
 	}
+	
+	for (tmp_list = exclude_path_list; tmp_list; tmp_list = tmp_list->next) {
+		g_free (tmp_list->data);
+	}
 	g_slist_free (exclude_path_list);
 	
-	return FALSE;
+	return results;
 }
 
 gboolean
@@ -670,123 +698,406 @@ gsearchtool_strdup_strftime (const gchar * format,
 	return result;
 }
 
-const char *
-get_file_type_for_mime_type (const gchar * file, 
-                             const gchar * mime)
+gchar *
+get_file_type_description (const gchar * file, 
+                           GnomeVFSFileInfo * file_info)
 {
-	const char * desc;
+	gchar * desc;
 	
-	if (file == NULL || mime == NULL) {
-		return gnome_vfs_mime_get_description (GNOME_VFS_MIME_TYPE_UNKNOWN);
+	if (file == NULL || file_info->mime_type == NULL) {
+		return g_strdup (gnome_vfs_mime_get_description (GNOME_VFS_MIME_TYPE_UNKNOWN));
 	}
 
-	desc = gnome_vfs_mime_get_description (mime);
+	desc = g_strdup (gnome_vfs_mime_get_description (file_info->mime_type));
 
-	if (g_file_test (file, G_FILE_TEST_IS_SYMLINK)) {
+	if (file_info->symlink_name != NULL) {
 	
-		GnomeVFSFileInfo *file_info;
-		gchar *absolute_symlink = NULL;
+		gchar * absolute_symlink = NULL;
+		gchar * str = NULL;
 		
-		file_info = gnome_vfs_file_info_new ();
-		gnome_vfs_get_file_info (file, file_info, GNOME_VFS_FILE_INFO_DEFAULT);
-		
-		if (file_info->symlink_name != NULL) {
-			if (g_path_is_absolute (file_info->symlink_name) != TRUE) {
-				gchar *dirname;
+		if (g_path_is_absolute (file_info->symlink_name) != TRUE) {
+			gchar *dirname;
 			
-				dirname = g_path_get_dirname (file);
-				absolute_symlink = g_strconcat (dirname, G_DIR_SEPARATOR_S, file_info->symlink_name, NULL);
-				g_free (dirname);
-			}
-			else {
-				absolute_symlink = g_strdup (file_info->symlink_name);
-			}
+			dirname = g_path_get_dirname (file);
+			absolute_symlink = g_strconcat (dirname, G_DIR_SEPARATOR_S, file_info->symlink_name, NULL);
+			g_free (dirname);
+		}
+		else {
+			absolute_symlink = g_strdup (file_info->symlink_name);
 		}
 		
 		if (g_file_test (absolute_symlink, G_FILE_TEST_EXISTS) != TRUE) {
-                       if ((g_ascii_strcasecmp (mime, "x-special/socket") != 0) &&
-                           (g_ascii_strcasecmp (mime, "x-special/fifo") != 0)) {
-				gnome_vfs_file_info_unref (file_info);
+                       if ((g_ascii_strcasecmp (file_info->mime_type, "x-special/socket") != 0) &&
+                           (g_ascii_strcasecmp (file_info->mime_type, "x-special/fifo") != 0)) {
 				g_free (absolute_symlink);
-				return _("link (broken)");
+				g_free (desc);
+				return g_strdup (_("link (broken)"));
 			}
 		}
 			
-		gnome_vfs_file_info_unref (file_info);
+		str = g_strdup_printf (_("link to %s"), (desc != NULL) ? desc : file_info->mime_type);
 		g_free (absolute_symlink);
-		return g_strdup_printf (_("link to %s"), (desc != NULL) ? desc : mime);
+		g_free (desc);
+		return str;
 	}
 	return desc;
 } 
 
+static gchar *
+gsearchtool_pixmap_file (const gchar * partial_path)
+{
+	gchar * path;
+
+	path = g_build_filename (DATADIR "/pixmaps/gsearchtool", partial_path, NULL);
+	if (g_file_test (path, G_FILE_TEST_EXISTS)) {
+		return path;
+	}
+	g_free (path);
+	return NULL;
+}
+
+static GdkPixbuf *
+gsearchtool_load_thumbnail_frame (void)
+{
+	GdkPixbuf * pixbuf = NULL;
+	gchar * image_path;
+	
+	image_path = gsearchtool_pixmap_file ("thumbnail_frame.png");
+
+	if (image_path != NULL) {
+		pixbuf = gdk_pixbuf_new_from_file (image_path, NULL);
+	}
+	g_free (image_path);
+	return pixbuf;
+} 
+
+static void
+gsearchtool_draw_frame_row (GdkPixbuf * frame_image, 
+                            gint target_width, 
+                            gint source_width, 
+                            gint source_v_position, 
+                            gint dest_v_position, 
+                            GdkPixbuf * result_pixbuf, 
+                            gint left_offset, 
+                            gint height)
+{
+	gint remaining_width;
+	gint h_offset;
+	gint slab_width;
+	
+	remaining_width = target_width;
+	h_offset = 0;
+	while (remaining_width > 0) {	
+		slab_width = remaining_width > source_width ? source_width : remaining_width;
+		gdk_pixbuf_copy_area (frame_image, left_offset, source_v_position, slab_width, 
+		                      height, result_pixbuf, left_offset + h_offset, dest_v_position);
+		remaining_width -= slab_width;
+		h_offset += slab_width; 
+	}
+}
+
+static void
+gsearchtool_draw_frame_column (GdkPixbuf * frame_image, 
+                               gint target_height, 
+                               gint source_height, 
+                               gint source_h_position, 
+                               gint dest_h_position, 
+                               GdkPixbuf * result_pixbuf, 
+                               gint top_offset, 
+                               gint width)
+{
+	gint remaining_height;
+	gint v_offset;
+	gint slab_height;
+	
+	remaining_height = target_height;
+	v_offset = 0;
+	while (remaining_height > 0) {	
+		slab_height = remaining_height > source_height ? source_height : remaining_height;
+		gdk_pixbuf_copy_area (frame_image, source_h_position, top_offset, width, slab_height, 
+		                      result_pixbuf, dest_h_position, top_offset + v_offset);
+		remaining_height -= slab_height;
+		v_offset += slab_height; 
+	}
+}
+
+static GdkPixbuf *
+gsearchtool_stretch_frame_image (GdkPixbuf *frame_image, 
+                                 gint left_offset, 
+                                 gint top_offset, 
+                                 gint right_offset, 
+                                 gint bottom_offset,
+                                 gint dest_width, 
+                                 gint dest_height, 
+                                 gboolean fill_flag)
+{
+	GdkPixbuf * result_pixbuf;
+	guchar * pixels_ptr;
+	gint frame_width, frame_height;
+	gint y, row_stride;
+	gint target_width, target_frame_width;
+	gint target_height, target_frame_height;
+	
+	frame_width = gdk_pixbuf_get_width (frame_image);
+	frame_height = gdk_pixbuf_get_height (frame_image);
+	
+	if (fill_flag) {
+		result_pixbuf = gdk_pixbuf_scale_simple (frame_image, dest_width, dest_height, GDK_INTERP_NEAREST);
+	} else {
+		result_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, dest_width, dest_height);
+	}
+	row_stride = gdk_pixbuf_get_rowstride (result_pixbuf);
+	pixels_ptr = gdk_pixbuf_get_pixels (result_pixbuf);
+	
+	/* clear the new pixbuf */
+	if (fill_flag == FALSE) {
+		for (y = 0; y < dest_height; y++) {
+			art_rgb_run_alpha (pixels_ptr, 255, 255, 255, 255, dest_width);
+			pixels_ptr += row_stride; 
+		}
+	}
+	
+	target_width  = dest_width - left_offset - right_offset;
+	target_frame_width = frame_width - left_offset - right_offset;
+	
+	target_height  = dest_height - top_offset - bottom_offset;
+	target_frame_height = frame_height - top_offset - bottom_offset;
+	
+	/* Draw the left top corner  and top row */
+	gdk_pixbuf_copy_area (frame_image, 0, 0, left_offset, top_offset, result_pixbuf, 0,  0);
+	gsearchtool_draw_frame_row (frame_image, target_width, target_frame_width, 0, 0, 
+	                            result_pixbuf, left_offset, top_offset);
+	
+	/* Draw the right top corner and left column */
+	gdk_pixbuf_copy_area (frame_image, frame_width - right_offset, 0, right_offset, top_offset, 
+	                      result_pixbuf, dest_width - right_offset,  0);
+	gsearchtool_draw_frame_column (frame_image, target_height, target_frame_height, 0, 0, 
+	                               result_pixbuf, top_offset, left_offset);
+
+	/* Draw the bottom right corner and bottom row */
+	gdk_pixbuf_copy_area (frame_image, frame_width - right_offset, frame_height - bottom_offset, 
+	                      right_offset, bottom_offset, result_pixbuf, dest_width - right_offset,  
+			      dest_height - bottom_offset);
+	gsearchtool_draw_frame_row (frame_image, target_width, target_frame_width, 
+	                            frame_height - bottom_offset, dest_height - bottom_offset, 
+				    result_pixbuf, left_offset, bottom_offset);
+		
+	/* Draw the bottom left corner and the right column */
+	gdk_pixbuf_copy_area (frame_image, 0, frame_height - bottom_offset, left_offset, bottom_offset, 
+	                      result_pixbuf, 0,  dest_height - bottom_offset);
+	gsearchtool_draw_frame_column (frame_image, target_height, target_frame_height, 
+	                               frame_width - right_offset, dest_width - right_offset, 
+				       result_pixbuf, top_offset, right_offset);
+	return result_pixbuf;
+}
+
+static GdkPixbuf *
+gsearchtool_embed_image_in_frame (GdkPixbuf * source_image, 
+                                  GdkPixbuf * frame_image, 
+                                  gint left_offset, 
+                                  gint top_offset, 
+                                  gint right_offset, 
+                                  gint bottom_offset)
+{
+	GdkPixbuf * result_pixbuf;
+	gint source_width, source_height;
+	gint dest_width, dest_height;
+	
+	source_width = gdk_pixbuf_get_width (source_image);
+	source_height = gdk_pixbuf_get_height (source_image);
+
+	dest_width = source_width + left_offset + right_offset;
+	dest_height = source_height + top_offset + bottom_offset;
+	
+	result_pixbuf = gsearchtool_stretch_frame_image (frame_image, left_offset, top_offset, right_offset, bottom_offset, 
+						         dest_width, dest_height, FALSE);
+		
+	gdk_pixbuf_copy_area (source_image, 0, 0, source_width, source_height, result_pixbuf, left_offset, top_offset);
+
+	return result_pixbuf;
+}
+
+static void
+gsearchtool_thumbnail_frame_image (GdkPixbuf ** pixbuf)
+{
+	GdkPixbuf * pixbuf_with_frame;
+	GdkPixbuf * frame;
+
+	frame = gsearchtool_load_thumbnail_frame ();
+	if (frame == NULL) {
+		return;
+	}
+	
+	pixbuf_with_frame = gsearchtool_embed_image_in_frame (*pixbuf, frame, 3, 3, 6, 6);
+	g_object_unref (*pixbuf);
+	g_object_unref (frame);
+	
+	*pixbuf = pixbuf_with_frame;
+}
+
+static GdkPixbuf *
+gsearchtool_get_thumbnail_image (const gchar * file)
+{
+	GdkPixbuf * pixbuf = NULL;
+	gchar * thumbnail_path;
+	gchar * uri;
+	
+	uri = gnome_vfs_get_uri_from_local_path (file);
+	thumbnail_path = gnome_thumbnail_path_for_uri (uri, GNOME_THUMBNAIL_SIZE_NORMAL);
+	
+	if (thumbnail_path != NULL) { 
+		if (g_file_test (thumbnail_path, G_FILE_TEST_EXISTS)) {
+			
+			GdkPixbuf * thumbnail_pixbuf = NULL;
+			gfloat scale_factor_x = 1.0;
+			gfloat scale_factor_y = 1.0;
+			gint scale_x;
+			gint scale_y;
+			
+			thumbnail_pixbuf = gdk_pixbuf_new_from_file (thumbnail_path, NULL);
+			gsearchtool_thumbnail_frame_image (&thumbnail_pixbuf);
+
+			if (gdk_pixbuf_get_width (thumbnail_pixbuf) > ICON_SIZE) {
+				scale_factor_x = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_width (thumbnail_pixbuf);
+			}
+			if (gdk_pixbuf_get_height (thumbnail_pixbuf) > ICON_SIZE) {
+				scale_factor_y = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_height (thumbnail_pixbuf);
+			}
+				
+			if (gdk_pixbuf_get_width (thumbnail_pixbuf) > gdk_pixbuf_get_height (thumbnail_pixbuf)) {
+				scale_x = ICON_SIZE;
+				scale_y = (gint) (gdk_pixbuf_get_height (thumbnail_pixbuf) * scale_factor_x);
+			}
+			else {
+				scale_x = (gint) (gdk_pixbuf_get_width (thumbnail_pixbuf) * scale_factor_y);
+				scale_y = ICON_SIZE;
+			}
+
+			pixbuf = gdk_pixbuf_scale_simple (thumbnail_pixbuf, scale_x, scale_y, GDK_INTERP_BILINEAR);
+			g_object_unref (thumbnail_pixbuf);
+		}
+		g_free (thumbnail_path);
+	}
+	g_free (uri);
+	return pixbuf;
+}
+
+static gchar *
+gsearchtool_icon_lookup (GSearchWindow * gsearch,
+                         const gchar * file, 
+                         const gchar * mime,
+                         GnomeVFSFileInfo * file_info)
+{
+	GnomeThumbnailFactory * thumbnail_factory = NULL;
+	GnomeIconLookupFlags lookup_flags = GNOME_ICON_LOOKUP_FLAGS_NONE;
+	gchar * icon_name = NULL;
+	gchar * uri;
+
+	uri = gnome_vfs_get_uri_from_local_path (file);
+
+	if (gsearch->show_thumbnails == TRUE) {
+		if ((strncmp (file_info->mime_type, "image/", 6) != 0) ||
+	    	    (file_info->size < gsearch->show_thumbnails_file_size_limit)) {
+		    	thumbnail_factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+			lookup_flags = GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES | 
+			               GNOME_ICON_LOOKUP_FLAGS_ALLOW_SVG_AS_THEMSELVES;
+		}
+	}
+
+	icon_name = gnome_icon_lookup (gtk_icon_theme_get_default (), 
+	                               thumbnail_factory,
+				       uri,
+				       NULL, 
+				       file_info,
+				       mime, 
+				       lookup_flags, 
+				       NULL);
+	
+	if (thumbnail_factory != NULL) {
+		g_object_unref (thumbnail_factory);
+	}
+	g_free (uri);
+	return icon_name;
+}
+
 GdkPixbuf *
-get_file_pixbuf_for_mime_type (GHashTable * hash,
-                               const gchar * file, 
-			       const gchar * mime) 
+get_file_pixbuf (GSearchWindow * gsearch,
+                 const gchar * file, 
+                 GnomeVFSFileInfo * file_info) 
 {
 	GdkPixbuf * pixbuf;
 	gchar * icon_name = NULL;
 	
-	if (file == NULL || mime == NULL) {
+	if (file == NULL || file_info->mime_type == NULL) {
 		icon_name = g_strdup (ICON_THEME_REGULAR_ICON);
 	}
 	else if ((g_file_test (file, G_FILE_TEST_IS_EXECUTABLE)) &&
-	         (g_ascii_strcasecmp (mime, "application/x-executable-binary") == 0)) {
+	         (g_ascii_strcasecmp (file_info->mime_type, "application/x-executable-binary") == 0)) {
 		icon_name = g_strdup (ICON_THEME_EXECUTABLE_ICON);
 	}
-	else if (g_ascii_strcasecmp (mime, "x-special/device-char") == 0) {
+	else if (g_ascii_strcasecmp (file_info->mime_type, "x-special/device-char") == 0) {
 		icon_name = g_strdup (ICON_THEME_CHAR_DEVICE);
 	}
-	else if (g_ascii_strcasecmp (mime, "x-special/device-block") == 0) {
+	else if (g_ascii_strcasecmp (file_info->mime_type, "x-special/device-block") == 0) {
 		icon_name = g_strdup (ICON_THEME_BLOCK_DEVICE);
 	}
-	else if (g_ascii_strcasecmp (mime, "x-special/socket") == 0) {
+	else if (g_ascii_strcasecmp (file_info->mime_type, "x-special/socket") == 0) {
 		icon_name = g_strdup (ICON_THEME_SOCKET);
 	}
-	else if (g_ascii_strcasecmp (mime, "x-special/fifo") == 0) {
+	else if (g_ascii_strcasecmp (file_info->mime_type, "x-special/fifo") == 0) {
 		icon_name = g_strdup (ICON_THEME_FIFO);
 	}
 	else {
-		icon_name = gnome_icon_lookup (gtk_icon_theme_get_default (), NULL, file, NULL, 
-					       NULL, mime, 0, NULL);
+		icon_name = gsearchtool_icon_lookup (gsearch, file, file_info->mime_type, file_info);
 	}
 	
-	pixbuf = (GdkPixbuf *) g_hash_table_lookup (hash, icon_name);
+	pixbuf = (GdkPixbuf *) g_hash_table_lookup (gsearch->search_results_filename_hash_table, icon_name);
 	
 	if (pixbuf == NULL) {
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name, 
-		                                   ICON_SIZE, 0, NULL);
-		g_hash_table_insert (hash, &icon_name, pixbuf);
-	}
-						  
-	g_free (icon_name);
-	return pixbuf;
-}
 
-gboolean
-is_nautilus_running (void)
-{
-	CORBA_Environment ev;
-	CORBA_Object obj;
-	gboolean ret;
-	
-	CORBA_exception_init (&ev); 
-	obj = bonobo_activation_activate_from_id ("OAFIID:Nautilus_Factory",
-		Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, NULL, &ev);
+		if (gsearch->show_thumbnails == TRUE) { 
+
+			if ((strncmp (file_info->mime_type, "image/", 6) != 0) ||
+	    		    (file_info->size < gsearch->show_thumbnails_file_size_limit)) {
 		
-	ret = !CORBA_Object_is_nil (obj, &ev);
-	
-	CORBA_Object_release (obj, &ev);	
-	CORBA_exception_free (&ev);
-	
-	return ret;
+				if (strcmp (icon_name, file) == 0) {
+					pixbuf = gdk_pixbuf_new_from_file_at_scale (file, ICON_SIZE, ICON_SIZE, TRUE, NULL);
+				}
+			
+				if (pixbuf == NULL) {
+					pixbuf = gsearchtool_get_thumbnail_image (file);
+				}
+			}
+		}
+
+		if (pixbuf == NULL) {
+			pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name, 
+			                                   ICON_SIZE, 0, NULL);
+		}
+		
+		if (pixbuf != NULL) {
+			g_hash_table_insert (gsearch->search_results_filename_hash_table, icon_name, pixbuf);
+		}
+		else {
+			g_free (icon_name);
+		}
+	}
+	else {
+		g_free (icon_name);
+	}					  
+
+	return pixbuf;
 }
 
 gboolean
 open_file_with_nautilus (GtkWidget * window,
                          const gchar * file)
 {
+	GnomeDesktopItem * ditem;
+ 	GdkScreen * screen;
+	GError *error = NULL;
 	gchar * command;
+	gchar * contents;
 	gchar * escaped;
 
 	escaped = g_shell_quote (file);
@@ -797,11 +1108,47 @@ open_file_with_nautilus (GtkWidget * window,
 	                       "--no-default-window ",
 	                       escaped, 
 	                       NULL);
-	gdk_x11_window_set_user_time (window->window, 0);
-	g_spawn_command_line_async (command, NULL);
+			       
+	contents = g_strdup_printf ("[Desktop Entry]\n"
+				    "Name=Nautilus\n"
+				    "Icon=file-manager\n"
+				    "Exec=%s\n"
+				    "Terminal=false\n"
+				    "StartupNotify=true\n"
+				    "Type=Application\n",
+				    command);
+
+	ditem = gnome_desktop_item_new_from_string (NULL,
+	                                            contents,
+	                                            strlen (contents),
+	                                            GNOME_DESKTOP_ITEM_LOAD_NO_TRANSLATIONS ,
+	                                            NULL);
+	
+	if (ditem == NULL) {
+		g_free (contents);
+		g_free (command);
+		g_free (escaped);
+		return FALSE;
+	}
+	
+	screen = gtk_widget_get_screen (window);
+
+	gnome_desktop_item_set_launch_time (ditem, 
+	                                    gtk_get_current_event_time ());
+
+	gnome_desktop_item_launch_on_screen (ditem, NULL, 
+	                                     GNOME_DESKTOP_ITEM_LAUNCH_ONLY_ONE, 
+					     screen, -1, &error);				
+
+	gnome_desktop_item_unref (ditem);
+	g_free (contents);
 	g_free (command);
 	g_free (escaped);
 
+	if (error) {		
+		g_error_free (error);
+		return FALSE;
+	}
 	return TRUE;
 }
 
@@ -844,7 +1191,7 @@ open_file_with_application (GtkWidget * window,
 				 	screen = gtk_widget_get_screen (window);
 					gnome_desktop_item_set_launch_time (ditem, gtk_get_current_event_time ());
 					gnome_desktop_item_launch_on_screen (ditem, uris, 
-						GNOME_DESKTOP_ITEM_LAUNCH_APPEND_URIS, screen, -1, &error);
+						GNOME_DESKTOP_ITEM_LAUNCH_APPEND_PATHS, screen, -1, &error);
 					if (error) {
 						result = FALSE;			
 						g_error_free (error);
