@@ -30,7 +30,6 @@
 #include "log_repaint.h"
 #include "logrtns.h"
 #include "info.h"
-#include "zoom.h"
 #include "monitor.h"
 #include "about.h"
 #include "desc_db.h"
@@ -43,6 +42,7 @@ static GObjectClass *parent_class;
 static GSList *logview_windows = NULL;
 static GConfClient *client = NULL;
 static UserPrefsStruct *user_prefs;
+static gboolean restoration_complete = FALSE;
 
 #define APP_NAME _("System Log Viewer")
 
@@ -61,10 +61,15 @@ static void open_databases (void);
 GtkWidget *logview_create_monitor_widget (LogviewWindow *window);
 static GtkWidget *logview_create_window (void);
 static void logview_menu_item_set_state (LogviewWindow *logviewwindow, char *path, gboolean state);
+static void toggle_sidebar (GtkAction *action, GtkWidget *callback_data);
 static void toggle_calendar (GtkAction *action, GtkWidget *callback_data);
-static void toggle_zoom (GtkAction *action, GtkWidget *callback_data);
-static void toggle_collapse_rows (GtkAction *action, GtkWidget *callback_data);
+static void logview_collapse_rows (GtkAction *action, GtkWidget *callback_data);
 static void toggle_monitor (GtkAction *action, GtkWidget *callback_data);
+static void logview_set_fontsize (LogviewWindow *logview);
+static void logview_bigger_text (GtkAction *action, GtkWidget *callback_data);
+static void logview_smaller_text (GtkAction *action, GtkWidget *callback_data);
+static void logview_normal_text (GtkAction *action, GtkWidget *callback_data);
+static void logview_calendar_set_state (LogviewWindow *LogviewWindow);
 static void logview_menus_set_state (LogviewWindow *logviewwindow);
 static void logview_search (GtkAction *action, GtkWidget *callback_data);
 static void logview_help (GtkAction *action, GtkWidget *parent_window);
@@ -74,6 +79,10 @@ static int logview_count_logs (void);
 gboolean window_size_changed_cb (GtkWidget *widget, GdkEventConfigure *event, 
 				 gpointer data);
 GType logview_window_get_type (void);
+static void loglist_add_log (LogviewWindow *window, Log *log);
+Log *logview_find_log_from_name (LogviewWindow *logview, gchar *name);
+static void loglist_select_log_from_name (LogviewWindow *logview, gchar *logname);
+
 
 /*
  *    ,-------.
@@ -103,10 +112,17 @@ static GtkActionEntry entries[] = {
 	{ "Search", GTK_STOCK_FIND, N_("_Find..."), "<control>F", N_("Find pattern in logs"),
 	  G_CALLBACK (logview_search) },
 
-	{"CollapseAll", NULL, N_("Collapse _All"), NULL, N_("Collapse all the rows"),
-	 G_CALLBACK (toggle_collapse_rows) },
+	{"ViewZoomIn", GTK_STOCK_ZOOM_IN, NULL, "<control>plus", N_("Bigger text size"),
+	 G_CALLBACK (logview_bigger_text)},
+	{"ViewZoomOut", GTK_STOCK_ZOOM_OUT, NULL, "<control>minus", N_("Smaller text size"),
+	 G_CALLBACK (logview_smaller_text)},
+	{"ViewZoom100", GTK_STOCK_ZOOM_100, NULL, "<control>0", N_("Normal text size"),
+	 G_CALLBACK (logview_normal_text)},
 
-       	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Open the help contents for the log viewer"), 
+	{"CollapseAll", NULL, N_("Collapse _All"), NULL, N_("Collapse all the rows"),
+	 G_CALLBACK (logview_collapse_rows) },
+
+	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Open the help contents for the log viewer"), 
 	  G_CALLBACK (logview_help) },
 	{ "AboutAction", GTK_STOCK_ABOUT, N_("_About"), NULL, N_("Show the about dialog for the log viewer"), 
 	  G_CALLBACK (AboutShowWindow) },
@@ -114,12 +130,12 @@ static GtkActionEntry entries[] = {
 };
 
 static GtkToggleActionEntry toggle_entries[] = {
+	{ "ShowSidebar", NULL, N_("Sidebar"), NULL, N_("Show the sidebar"), 
+		G_CALLBACK (toggle_sidebar), TRUE },
 	{ "MonitorLogs", NULL, N_("_Monitor"), "<control>M", N_("Monitor Current Log"),
 	  G_CALLBACK (toggle_monitor) },
 	{"ShowCalendar", NULL,  N_("Ca_lendar"), "<control>L", N_("Show Calendar Log"), 
-	 G_CALLBACK (toggle_calendar) },
-	{"ShowDetails", NULL,  N_("_Entry Detail"), "<control>E", N_("Show Entry Detail"), 
-	 G_CALLBACK (toggle_zoom) },
+	 G_CALLBACK (toggle_calendar), TRUE },
 };
 
 static const char *ui_description = 
@@ -142,10 +158,14 @@ static const char *ui_description =
 	"			<menuitem action='Search'/>"
 	"		</menu>"	
 	"		<menu action='ViewMenu'>"
+  "     <menuitem action='ShowSidebar'/>"
 	"			<menuitem action='ShowCalendar'/>"
-	"			<menuitem action='ShowDetails'/>"
 	"                       <separator/>"
 	"			<menuitem action='CollapseAll'/>"
+  "                       <separator/>"
+  "     <menuitem action='ViewZoomIn'/>"
+  "     <menuitem action='ViewZoomOut'/>"
+  "     <menuitem action='ViewZoom100'/>"
 	"		</menu>"
 	"		<menu action='HelpMenu'>"
 	"			<menuitem action='HelpContents'/>"
@@ -169,6 +189,23 @@ GOptionEntry options[] = {
 	{ NULL }
 };
 
+void
+logview_save_prefs (LogviewWindow *logview)
+{
+  GList *list;
+  Log *log;
+
+	prefs_free_loglist(user_prefs);
+  for (list = logview->logs; list != NULL; list = g_list_next (list)) {
+      log = list->data;
+      user_prefs->logs = g_slist_append (user_prefs->logs, log->name);
+	}
+  user_prefs->logfile = logview->curlog->name;
+	user_prefs->fontsize = logview->fontsize;
+	if (restoration_complete)
+		prefs_save (client, user_prefs);
+}
+
 /* ----------------------------------------------------------------------
    NAME:          destroy
    DESCRIPTION:   Exit program.
@@ -179,7 +216,7 @@ destroy (GObject *object, gpointer data)
 {
    LogviewWindow *window = data;
    logview_windows = g_slist_remove (logview_windows, window);
-   if (window->monitored)
+   if (window->curlog->monitored)
 	   monitor_stop (window);
    if (logview_windows == NULL) {
 	   if (window->curlog && !(window->curlog->display_name))
@@ -229,25 +266,60 @@ die (GnomeClient *gnome_client, gpointer client_data)
 }
 
 void
-logview_create_window_open_file (gchar *file)
+logview_select_log (LogviewWindow *logview, Log *log)
+{
+	g_return_if_fail (LOGVIEW_IS_WINDOW (logview));
+	g_return_if_fail (log);
+	
+	logview->curlog = log;
+	logview_menus_set_state (logview);
+	logview_calendar_set_state (logview);
+	logview_update_version_bar (logview);
+	log_repaint (logview);
+	logview_save_prefs (logview);
+} 
+
+void
+logview_add_log (LogviewWindow *logview, Log *log)
+{
+  logview->logs = g_list_append (logview->logs, log);
+  loglist_add_log (logview, log);
+  log->monitored = FALSE;
+  log->first_time = TRUE;
+} 
+
+void
+logview_add_log_from_name (LogviewWindow *logview, gchar *file)
 {
 	Log *log;
-	log = OpenLogFile (file);
+	log = OpenLogFile (file, TRUE);
 	if (log != NULL) {
-		GtkWidget *window;
-		window = logview_create_window ();
-		if (window) {
-			LogviewWindow *logview;
-			logview = LOGVIEW_WINDOW (window);
-			logview->curlog = log;
-			logview->monitored = FALSE;
-			logview_menus_set_state (logview);
-			log_repaint (logview);
-			gtk_widget_show (window);
+		  logview_add_log (logview, log);
+	}
+}
+
+void
+logview_add_logs_from_names (LogviewWindow *logview, GSList *lognames)
+{
+	GSList *list;
+	Log *log;
+	
+	for (list = lognames; list != NULL; list = g_slist_next (list)) {
+		if (logview_find_log_from_name (logview, list->data) == NULL) {
+			log = OpenLogFile (list->data, FALSE);
+			if (log != NULL) {
+				logview_add_log (logview, log);
+			}
 		}
 	}
 }
 
+logview_remove_log (LogviewWindow *logview, Log *log)
+{
+	CloseLog (log);
+	logview->logs = g_list_remove (logview->logs, log);
+}
+
 GOptionContext *
 logview_init_options ()
 {
@@ -271,6 +343,7 @@ main (int argc, char *argv[])
    GnomeClient *gnome_client;
    GError *error;
    GnomeProgram *program;
+	 LogviewWindow *logview;
    int i;
 
    bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -296,21 +369,19 @@ main (int argc, char *argv[])
    context = logview_init_options ();
    g_option_context_parse (context, &argc, &argv, &error);
 
-   /* Open a new window for each log passed as a parameter */
-   /* If no log was passed as parameter, open regular logs */
+   /* Open regular logs and add each log passed as a parameter */
+
+	 logview = LOGVIEW_WINDOW(logview_create_window ());
+	 logview_add_logs_from_names (logview, user_prefs->logs);
+	 loglist_select_log_from_name (logview, user_prefs->logfile);
    if (argc > 1) {
 	   for (i=1; i<argc; i++) {
 		   file_to_open = argv[i];
-		   logview_create_window_open_file (file_to_open);
+			 logview_add_log_from_name (logview, file_to_open);
 	   }
-   } else
-	   logview_create_window_open_file (user_prefs->logfile);
-
-   if (logview_count_logs() == 0) {
-	   GtkWidget *window;
-	   window = logview_create_window ();
-	   gtk_widget_show (window);
    }
+	 gtk_widget_show (GTK_WIDGET(logview));
+	 restoration_complete = TRUE;
 
    gnome_client = gnome_master_client ();
 
@@ -336,35 +407,237 @@ main (int argc, char *argv[])
 static GtkWidget *
 logview_create_window ()
 {
-   LogviewWindow *logviewwindow;
+   LogviewWindow *logview;
    GtkWidget *window;
 
    regexp_db = NULL;
 
    /*  Display main window */
    window = g_object_new (LOGVIEW_TYPE_WINDOW, NULL);
-   logviewwindow = LOGVIEW_WINDOW (window);
+   logview = LOGVIEW_WINDOW (window);
 
-   logviewwindow->loginfovisible = FALSE;
-   logviewwindow->zoom_visible = FALSE;
-   logviewwindow->zoom_scrolled_window = NULL;
-   logviewwindow->zoom_dialog = NULL;
-   logviewwindow->clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (window),
+	 logview->sidebar_visible = TRUE;
+	 logview->calendar_visible = TRUE;
+   logview->loginfovisible = FALSE;
+   logview->logs = NULL;
+   logview->clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (window),
 							     GDK_SELECTION_CLIPBOARD);
 
    /* FIXME : we need to listen to this key, not just read it. */
-   gtk_ui_manager_set_add_tearoffs (logviewwindow->ui_manager, 
-				    gconf_client_get_bool 
-				    (client, "/desktop/gnome/interface/menus_have_tearoff", NULL));
+   gtk_ui_manager_set_add_tearoffs (logview->ui_manager, 
+																		gconf_client_get_bool 
+																		(client, "/desktop/gnome/interface/menus_have_tearoff", NULL));
 
    g_signal_connect (GTK_OBJECT (window), "destroy",
-		     G_CALLBACK (destroy), logviewwindow);
+										 G_CALLBACK (destroy), logview);
 
    logview_windows = g_slist_prepend (logview_windows, window);
 
    return window;
 }
 
+Log *
+logview_find_log_from_name (LogviewWindow *logview, gchar *name)
+{
+  GList *list;
+  Log *log;
+  for (list = logview->logs; list != NULL; list = g_list_next (list)) {
+    log = list->data;
+    if (g_ascii_strncasecmp (log->name, name, 255) == 0) {
+      return log;
+    }
+  }
+  return NULL;
+}
+
+static void
+loglist_selection_changed (GtkTreeSelection *selection, LogviewWindow *logview)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gchar *name;
+  Log *log;
+  
+  g_return_if_fail (LOGVIEW_IS_WINDOW (logview));
+  g_return_if_fail (selection);
+
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+      /* there is no selected log right now */
+      logview->curlog = NULL;
+      logview_set_window_title (logview);
+      logview_menus_set_state (logview);
+      gtk_calendar_clear_marks (GTK_CALENDAR(logview->calendar));
+      log_repaint (logview);
+      return;
+  }
+  
+  gtk_tree_model_get (model, &iter, 0, &name, -1);
+  g_return_if_fail (name);
+
+  if ((g_str_has_prefix (name, "<b>")) && (strlen (name) > 7)) {
+      gchar *stripped;
+      stripped = g_strndup (name + 3, strlen(name)-7);
+      g_free (name);
+      name = stripped;
+  }
+
+  log = logview_find_log_from_name (logview, name);
+  g_free (name);
+  logview_select_log (logview, log);
+}
+
+GtkTreePath *
+loglist_find_logname (LogviewWindow *logview, gchar *logname)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	GtkTreePath *path = NULL;
+	gchar *name;
+
+	model =  gtk_tree_view_get_model (GTK_TREE_VIEW(logview->treeview));
+	if (!gtk_tree_model_get_iter_first (model, &iter))
+		return NULL;
+
+	do {
+		gtk_tree_model_get (model, &iter, 0, &name, -1);
+		if (g_ascii_strncasecmp (logname, name, 255) == 0) {
+			g_free (name);
+			path = gtk_tree_model_get_path (model, &iter);
+			return path;			
+		}
+		g_free (name);
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return NULL;
+}
+
+static void
+loglist_select_log_from_name (LogviewWindow *logview, gchar *logname)
+{
+	GtkTreePath *path;
+	path = loglist_find_logname (logview, logname);
+	if (path) {
+		GtkTreeSelection *selection;
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (logview->treeview));
+		gtk_tree_selection_select_path (selection, path);
+	}
+}
+
+static void 
+loglist_add_log (LogviewWindow *logview, Log *log)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (logview->treeview));
+    
+	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, log->name, -1);
+
+	if (restoration_complete) {
+		GtkTreeSelection *selection;
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (logview->treeview));
+		gtk_tree_selection_select_iter (selection, &iter);
+	}
+}
+
+static void
+loglist_remove_selected_log (LogviewWindow *logview)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreeSelection *selection;
+	gchar *name;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (logview->treeview));
+	gtk_tree_selection_get_selected (selection, &model, &iter);
+
+	gtk_tree_model_get (model, &iter, 0, &name, -1);
+	
+	if (name) {
+		Log *log;
+		
+		log = logview_find_log_from_name (logview, name);
+		logview_remove_log (logview, log);
+
+		if (gtk_list_store_remove (GTK_LIST_STORE (model), &iter))
+			gtk_tree_selection_select_iter (selection, &iter);
+	}
+}
+
+GtkWidget *
+loglist_create (LogviewWindow *window)
+{
+  GtkWidget *label;
+  GtkWidget *scrolled;
+  GtkWidget *vbox;
+  GtkWidget *treeview;
+  GtkListStore *model;
+  GtkTreeViewColumn *column;
+  GtkTreeSelection *selection;
+  GtkCellRenderer *cell;
+    
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_box_set_spacing (GTK_BOX (vbox), 3);
+  
+  scrolled = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
+				       GTK_SHADOW_ETCHED_IN);
+  
+  model = gtk_list_store_new (1, G_TYPE_STRING);
+  treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
+  g_object_unref (G_OBJECT (model));
+  
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+  g_signal_connect (G_OBJECT (selection), "changed",
+		    G_CALLBACK (loglist_selection_changed), window);
+
+  cell = gtk_cell_renderer_text_new ();
+  column = gtk_tree_view_column_new_with_attributes ("words", cell, "markup", 0, NULL);
+  gtk_tree_view_column_set_sort_column_id (column, 0);
+  
+  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+  
+  gtk_container_add (GTK_CONTAINER (scrolled), treeview);
+  gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
+
+  window->treeview = treeview;
+  gtk_widget_show (vbox);
+  return vbox;
+}
+
+void
+logview_version_selector_changed (GtkComboBox *version_selector, gpointer user_data)
+{
+	LogviewWindow *logview = user_data;
+	Log *log = logview->curlog;
+	int selected;
+
+	selected = gtk_combo_box_get_active (version_selector);
+
+	if (selected == log->current_version)
+		return;
+
+	/* select a new version */
+	if (selected == 0) {
+		logview_select_log (logview, log->parent_log);
+	} else {
+		Log *new;
+		if (log->parent_log) {
+			new = log->parent_log->older_logs[selected];
+		} else {
+			new = log->older_logs[selected];
+		}
+
+		logview_select_log (logview, new);
+	}
+}
+	
+
+
 /* ----------------------------------------------------------------------
    NAME:        CreateMainWin
    DESCRIPTION: Creates the main window.
@@ -383,6 +656,11 @@ CreateMainWin (LogviewWindow *window)
    GtkAccelGroup *accel_group;
    GError *error = NULL;
    GtkWidget *menubar;
+   GtkWidget *loglist;
+   GtkWidget *hpaned;
+	 GtkWidget *label;
+	 PangoFontDescription *fontdesc;
+	 PangoContext *context;
    const gchar *column_titles[] = { N_("Date"), N_("Host Name"),
                                     N_("Process"), N_("Message"), NULL };
 
@@ -413,8 +691,24 @@ CreateMainWin (LogviewWindow *window)
    menubar = gtk_ui_manager_get_widget (window->ui_manager, "/LogviewMenu");
    gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
 
-   window->main_view = gtk_frame_new (NULL);
-   gtk_box_pack_start (GTK_BOX (vbox), window->main_view, TRUE, TRUE, 0);
+   /* panes */
+   hpaned = gtk_hpaned_new ();
+   gtk_box_pack_start (GTK_BOX (vbox), hpaned, TRUE, TRUE, 0);
+
+   /* First pane : sidebar (list of logs + calendar) */
+	 window->sidebar = gtk_vbox_new (FALSE, 0);
+
+	 CalendarMenu (window);
+	 gtk_box_pack_end (GTK_BOX (window->sidebar), GTK_WIDGET(window->calendar), FALSE, FALSE, 0);
+
+	 window->loglist = loglist_create (window);
+	 gtk_box_pack_start (GTK_BOX (window->sidebar), window->loglist, TRUE, TRUE, 0);
+
+	 gtk_paned_pack1 (GTK_PANED (hpaned), window->sidebar, FALSE, FALSE);
+
+   /* Second pane : log and monitor */
+   window->main_view = gtk_vbox_new (FALSE, 0);
+   gtk_paned_pack2 (GTK_PANED (hpaned), GTK_WIDGET (window->main_view), TRUE, TRUE);
 
    /* Scrolled windows for the main view and monitor view */
    window->log_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -422,31 +716,40 @@ CreateMainWin (LogviewWindow *window)
                GTK_POLICY_AUTOMATIC,
                GTK_POLICY_AUTOMATIC);
    window->mon_scrolled_window = logview_create_monitor_widget (window);
-
-   /* We ref the two scrolled_windows so they are not destroyed when we remove one from
-    * the main_view to put the other */
-   g_object_ref (window->log_scrolled_window);
-   g_object_ref (window->mon_scrolled_window);
-
-   gtk_container_add (GTK_CONTAINER (window->main_view), window->log_scrolled_window);
+	 gtk_box_pack_start (GTK_BOX(window->main_view), window->log_scrolled_window, TRUE, TRUE, 0);
+	 gtk_box_pack_start (GTK_BOX(window->main_view), window->mon_scrolled_window, TRUE, TRUE, 0);
 
    /* Main Tree View */
-   tree_store = gtk_tree_store_new (4, G_TYPE_STRING, G_TYPE_STRING,
-				    G_TYPE_STRING, G_TYPE_STRING);
-
-   window->view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
+   window->view = gtk_tree_view_new ();
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (window->view), TRUE);
-   g_object_unref (G_OBJECT (tree_store)); 
    
    for (i = 0; column_titles[i]; i++) {
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes (_(column_titles[i]),
                     renderer, "text", i, NULL);
-        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE); 
+        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
         gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_append_column (GTK_TREE_VIEW (window->view), column);
    }
 
+	 /* Version selector */
+	 window->version_bar = gtk_hbox_new (FALSE, 0);
+	 gtk_container_set_border_width (GTK_CONTAINER (window->version_bar), 3);
+	 window->version_selector = gtk_combo_box_new_text ();
+	 g_signal_connect (G_OBJECT (window->version_selector), "changed",
+										 G_CALLBACK (logview_version_selector_changed), window);
+	 label = gtk_label_new (_("Version : "));
+
+	 gtk_box_pack_end (GTK_BOX(window->version_bar), window->version_selector, FALSE, FALSE, 0);
+	 gtk_box_pack_end (GTK_BOX(window->version_bar), label, FALSE, FALSE, 0);
+	 gtk_box_pack_end (GTK_BOX(window->main_view), window->version_bar, FALSE, FALSE, 0);
+
+	 /* Remember the original font size */
+	 context = gtk_widget_get_pango_context (window->view);
+	 fontdesc = pango_context_get_font_description (context);
+	 window->original_fontsize = pango_font_description_get_size (fontdesc) / PANGO_SCALE;
+	 window->fontsize = window->original_fontsize;		 
+
    gtk_container_add (GTK_CONTAINER (window->log_scrolled_window), GTK_WIDGET (window->view));
    gtk_widget_show_all (window->log_scrolled_window);
 
@@ -456,23 +759,25 @@ CreateMainWin (LogviewWindow *window)
    /* Add signal handlers */
    g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (handle_selection_changed_cb), window);
-   g_signal_connect (G_OBJECT (window->view), "row_activated",
-                     G_CALLBACK (handle_row_activation_cb), window);
+	 g_signal_connect (G_OBJECT (window->view), "row-expanded",
+										 G_CALLBACK (handle_row_expansion_cb), window);
+	 g_signal_connect (G_OBJECT (window->view), "row-collapsed",
+										 G_CALLBACK (handle_row_collapse_cb), window);
    g_signal_connect (G_OBJECT (window), "configure_event",
 		     G_CALLBACK (window_size_changed_cb), window);
 
-   window->find_bar = gtk_toolbar_new ();
-   logview_findbar_populate (window, window->find_bar);
-   gtk_toolbar_set_style (GTK_TOOLBAR (window->find_bar), GTK_TOOLBAR_BOTH_HORIZ);
-   gtk_box_pack_start (GTK_BOX (vbox), window->find_bar, FALSE, FALSE, 0);
+   window->find_bar = logview_findbar_new (window);
+	 gtk_box_pack_start (GTK_BOX (vbox), window->find_bar, FALSE, FALSE, 0);
    gtk_widget_show (window->find_bar);
 
    /* Status area at bottom */
    window->statusbar = gtk_statusbar_new ();
    gtk_box_pack_start (GTK_BOX (vbox), window->statusbar, FALSE, FALSE, 0);
 
-   gtk_widget_show_all (vbox);
+	 gtk_widget_show_all (vbox);
    gtk_widget_hide (window->find_bar);
+	 gtk_widget_hide (window->mon_scrolled_window);
+	 gtk_widget_hide (window->version_bar);
 }
 
 /* ----------------------------------------------------------------------
@@ -487,28 +792,13 @@ CloseLogMenu (GtkAction *action, GtkWidget *callback_data)
 
    g_return_if_fail (window->curlog);
 
-   if (window->monitored) {
+   if (window->curlog->monitored) {
 	   GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/FileMenu/MonitorLogs");
 	   gtk_action_activate (action);
    }
 
-   if (window->calendar_visible) {
-	   GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/ViewMenu/ShowCalendar");
-	   gtk_action_activate (action);
-   }
-
-   if (window->zoom_visible) {
-	   GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/ViewMenu/ShowDetails");
-	   gtk_action_activate (action);
-   }
-
    gtk_widget_hide (window->find_bar);
-
-   CloseLog (window->curlog);
-   window->curlog = NULL;
-   logview_menus_set_state (window);
-   logview_set_window_title (window);
-   log_repaint (window);
+	 loglist_remove_selected_log (window);
 }
 
 /* ----------------------------------------------------------------------
@@ -520,7 +810,7 @@ static void
 FileSelectResponse (GtkWidget * chooser, gint response, gpointer data)
 {
    char *f;
-   LogviewWindow *window = data;
+   LogviewWindow *logview = data;
 
    gtk_widget_hide (GTK_WIDGET (chooser));
    if (response != GTK_RESPONSE_OK)
@@ -530,36 +820,19 @@ FileSelectResponse (GtkWidget * chooser, gint response, gpointer data)
 
    if (f != NULL) {
 	   /* Check if the log is not already opened */
-	   GSList *list;
-	   LogviewWindow *w;
-	   for (list = logview_windows; list != NULL; list = g_slist_next (list)) {
-		   w = list->data;
-		   if (w->curlog == NULL)
-			   continue;
-		   if (g_ascii_strncasecmp (w->curlog->name, f, 255) == 0) {
-			   gtk_window_present (GTK_WINDOW(w));
+	   GList *list;
+		 Log *log;
+	   for (list = logview->logs; list != NULL; list = g_list_next (list)) {
+		   log = list->data;
+		   if (g_ascii_strncasecmp (log->name, f, 255) == 0) {
+				 loglist_select_log_from_name (logview, log->name);
 			   return;
 		   }
 	   }
 
-	   /* If a log was already opened in the window, open a new one... */
-	   if (window->curlog) {
-		   logview_create_window_open_file (f);		   
-	   } else {
-		   Log *tl;
-		   if ((tl = OpenLogFile (f)) != NULL) {
-			   window->curlog = tl;
-			   window->curlog->first_time = TRUE; 
-			   window->monitored = FALSE;
-			   
-			   log_repaint (window);
-			   
-			   if (window->calendar_visible)
-				   init_calendar_data(window);
-			   
-			   logview_menus_set_state (window);
-		   }
-	   }
+	   Log *tl;
+	   if ((tl = OpenLogFile (f, TRUE)) != NULL)
+	     logview_add_log (logview, tl);
    }
    
    g_free (f);
@@ -717,31 +990,37 @@ open_databases (void)
 
 }
 
+static void
+toggle_sidebar (GtkAction *action, GtkWidget *callback_data)
+{
+	LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
+
+	if (window->sidebar_visible)
+		gtk_widget_hide (window->sidebar);
+	else
+		gtk_widget_show (window->sidebar);
+	window->sidebar_visible = !(window->sidebar_visible);
+}
+
 static void 
 toggle_calendar (GtkAction *action, GtkWidget *callback_data)
 {
-    LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
-    if (window->calendar_visible) {
-	window->calendar_visible = FALSE;
-	gtk_widget_hide (window->calendar_dialog);
-    }
-    else
-	CalendarMenu (window);
-}
+	LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
 
-static void
-toggle_zoom (GtkAction *action, GtkWidget *callback_data)
-{
-    LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
-    if (window->zoom_visible) {
-	    window->zoom_visible = FALSE;
-	    gtk_widget_hide (window->zoom_dialog);
-    } else
-	    create_zoom_view (window);
+	if (window->calendar_visible)
+		gtk_widget_hide (window->calendar);
+	else {
+		gtk_widget_show (window->calendar);
+		if (!window->sidebar_visible) {
+			GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/ViewMenu/ShowSidebar");
+			gtk_action_activate (action);
+		}
+	}
+	window->calendar_visible = !(window->calendar_visible);
 }
 
 static void 
-toggle_collapse_rows (GtkAction *action, GtkWidget *callback_data)
+logview_collapse_rows (GtkAction *action, GtkWidget *callback_data)
 {
     LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
     gtk_tree_view_collapse_all (GTK_TREE_VIEW (window->view));
@@ -753,22 +1032,60 @@ toggle_monitor (GtkAction *action, GtkWidget *callback_data)
     LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
     g_return_if_fail (window->curlog);
     if (!window->curlog->display_name) {
-	    if (window->monitored) {
-		    gtk_container_remove (GTK_CONTAINER (window->main_view), window->mon_scrolled_window);
-		    gtk_container_add (GTK_CONTAINER (window->main_view), window->log_scrolled_window);
+	    if (window->curlog->monitored) {
+				gtk_widget_hide (window->mon_scrolled_window);
+				gtk_widget_show (window->log_scrolled_window);
 		    monitor_stop (window);
-		    window->monitored = FALSE;
+		    window->curlog->monitored = FALSE;
 	    } else {
-		    gtk_container_remove (GTK_CONTAINER(window->main_view), window->log_scrolled_window);
-		    gtk_container_add (GTK_CONTAINER(window->main_view), window->mon_scrolled_window);
+				gtk_widget_hide (window->log_scrolled_window);
+				gtk_widget_show (window->mon_scrolled_window);
 		    monitor_start (window);
-		    window->monitored = TRUE;
+		    window->curlog->monitored = TRUE;
 	    }
 	    logview_set_window_title (window);
 	    logview_menus_set_state (window);
     }
 }
 
+static void
+logview_set_fontsize (LogviewWindow *logview)
+{
+	PangoFontDescription *fontdesc;
+	PangoContext *context;
+	int i;
+	
+	context = gtk_widget_get_pango_context (logview->view);
+	fontdesc = pango_context_get_font_description (context);
+	pango_font_description_set_size (fontdesc, (logview->fontsize)*PANGO_SCALE);
+	gtk_widget_modify_font (logview->view, fontdesc);
+	logview_save_prefs (logview);
+}	
+
+static void
+logview_bigger_text (GtkAction *action, GtkWidget *callback_data)
+{
+	LogviewWindow *logview = LOGVIEW_WINDOW (callback_data);
+	logview->fontsize = MIN (logview->fontsize + 1, 24);
+	logview_set_fontsize (logview);
+}	
+
+static void
+logview_smaller_text (GtkAction *action, GtkWidget *callback_data)
+{
+	LogviewWindow *logview = LOGVIEW_WINDOW (callback_data);
+	logview->fontsize = MAX (logview->fontsize-1, 6);
+	logview_set_fontsize (logview);
+}	
+
+static void
+logview_normal_text (GtkAction *action, GtkWidget *callback_data)
+{
+	LogviewWindow *logview = LOGVIEW_WINDOW (callback_data);
+	logview->fontsize = logview->original_fontsize;
+	logview_set_fontsize (logview);
+}
+	
 static void
 logview_search (GtkAction *action,GtkWidget *callback_data)
 {
@@ -822,33 +1139,49 @@ logview_menu_item_set_state (LogviewWindow *logviewwindow, char *path, gboolean
 	gtk_action_set_sensitive (action, state);
 }
 
+static void
+logview_calendar_set_state (LogviewWindow *logview)
+{
+	if (logview->curlog->has_date) {
+		init_calendar_data (logview);
+		if (logview->calendar_visible)
+			gtk_widget_show (logview->calendar);
+	} else {
+		gtk_widget_hide (logview->calendar);
+	}
+}
+
 static void
 logview_menus_set_state (LogviewWindow *window)
 {
-	if (window->monitored) {
+	if (window->curlog && window->curlog->monitored) {
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Copy", FALSE);
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Search", FALSE);
-		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", FALSE);
-		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowDetails", FALSE); 
 		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/CollapseAll", FALSE);
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/SelectAll", FALSE);
 	} else {
 		if (window->curlog) {
+
 			if (window->curlog->display_name)
 				logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", FALSE);
 			else
 				logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", TRUE);
+
+            if (window->curlog->has_date)
+                logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", (window->curlog != NULL));
+            else
+                logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", FALSE);
+
 		} else
 			logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", FALSE);
 		
 		logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/Properties", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/CloseLog", (window->curlog != NULL));
-		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", (window->curlog != NULL));
-		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowDetails", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/CollapseAll", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Search", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Copy", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/SelectAll", (window->curlog != NULL));
+        logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", FALSE);
 	}
 }
 
@@ -890,7 +1223,7 @@ logview_set_window_title (LogviewWindow *window)
 		else
 			logname = window->curlog->name;
 		
-		if (window->monitored) 
+		if (window->curlog->monitored) 
 			window_title = g_strdup_printf (_("%s (monitored) - %s"), logname, APP_NAME);
 		else
 			window_title = g_strdup_printf ("%s - %s", logname, APP_NAME);
